// Library used to retrieve and set cookies

import API from './Api'
import config from '../../local/js/config'
import $ from 'jquery'
const cookies = require('browser-cookies')

export default class Storage {
  /**
     * Function used to save new Highlights, locally or externally.
     * @param {object} hgt The new highlight to be saved.
     */
  static saveHighlights (hgt) {
    // saves on the cookies
    this.saveLocal('hgt', hgt)
    if (window.navigator.onLine) {
      API.sendHighlight(hgt)
    }
  }

  /**
     * Function used to save new Comments, locally or externally.
     * @param {object} cmt            The new comment to be saved.
     */
  static saveComments (cmt) {
    // saves on the cookies
    this.saveLocal('cmt', cmt)
    if (window.navigator.onLine) {
      API.sendComment(cmt)
    }
  }

  /**
     * Function used to check if user sent a feedback.
     */
  static async retrieveFeedback (id) {
    if (window.navigator.onLine) {
      const response = await API.getFeedback(id)
      console.log(response)
      if (response !== 'NOT_FOUND') return response
      return false
    } else {
      return false
    }
  }

  static async saveFeedback (id, feedback) {
    if (await API.sendFeedback(id, feedback)) {
      this.saveLocal('feedback_' + id, feedback)
      $(document).trigger('successMessage', ['Feedback enviado com sucesso.'])
      return true
    } else {
      return false
    }
  }

  /**
     * Function used to retrieve existing highlights.
     * @return {array} Array containing all the highlights.
     */
  static async retrieveHighlights () {
    const local_hgts = await this.retrieveLocal('hgt')
    let list = local_hgts

    if (window.navigator.onLine) {
      const external_hgts = await API.getHighlights()
      if (external_hgts) {
        list = external_hgts
        if (window.localStorage) {
          window.localStorage.setItem((config.ModuleName.replace(/\s/g, '')) + '_' + 'hgt', JSON.stringify(list))
        } else {
          cookies.set((config.ModuleName.replace(/\s/g, '')) + '_' + 'hgt', JSON.stringify(list))
        }
      } else console.log('No highlights retrieved')
      return list
    } else {
      return list
    }
  }

  /**
     * Function used to retrieve existing comments.
     * @return {array} Array containing all the comments.
     */
  static async retrieveComments () {
    const local_cmts = await this.retrieveLocal('cmt')
    let list = local_cmts

    if (window.navigator.onLine) {
      const external_cmts = await API.getComments()
      if (external_cmts) {
        list = external_cmts
        if (window.localStorage) {
          window.localStorage.setItem((config.ModuleName.replace(/\s/g, '')) + '_' + 'cmt', JSON.stringify(list))
        } else {
          cookies.set((config.ModuleName.replace(/\s/g, '')) + '_' + 'cmt', JSON.stringify(list))
        }
      } else console.log('No highlights retrieved')
      return list
    } else {
      return list
    }
  }

  /**
     * Function used to save new Comments, locally or externally.
     * @param {int}    cmt             Id of the changed comment.
     * @param {string} comment        New comment content.
     */
  static updateComment (cmt, comment) {
    // saves on the cookies
    this.updateLocalComment(cmt, comment)
    if (window.navigator.onLine) {
      API.updateComment(cmt, comment)
    }
  }

  /**
     * Function used to save new Comments, locally or externally.
     * @param {int}    cmt             Id of the changed comment.
     * @param {string} comment         New comment content.
     */
  static async updateLocalComment (cmt, comment) {
    const type = 'cmt'
    const current = await this.retrieveLocal(type)
    if (!current) return true

    const Id = current.findIndex(x => x.hashid == cmt)
    current[Id].comment = comment

    const save = current

    if (window.localStorage) {
      window.localStorage.setItem((config.ModuleName.replace(/\s/g, '')) + '_' + type, JSON.stringify(save))
    } else {
      cookies.set((config.ModuleName.replace(/\s/g, '')) + '_' + type, JSON.stringify(save))
    }
  }

  /**
     * Function used to delete Highlights, locally and externally.
     * @param {Number} id - Id of the highlight to be deleted
     */
  static deleteHighlight (id) {
    // saves on the cookies
    this.deleteLocal('hgt', id)
    if (window.navigator.onLine) {
      // saves on the external DB
      API.deleteHighlight(id)
    }
  }

  /**
     * Function used to delete Highlights and Comments.
     * @param {Number} id - Id of the highlight to be deleted
     */
  static async deleteLocal (type, id) {
    const current = await this.retrieveLocal(type)
    if (!current) return true

    const Id = current.findIndex(x => x.hashid == id)
    current.splice(Id, 1)

    const save = current

    if (window.localStorage) {
      window.localStorage.setItem((config.ModuleName.replace(/\s/g, '')) + '_' + type, JSON.stringify(save))
    } else {
      cookies.set((config.ModuleName.replace(/\s/g, '')) + '_' + type, JSON.stringify(save))
    }
  }

  /**
     * Function used to deleteComments, locally and externally.
     */
  static deleteComment (id, hgt) {
    // saves on the cookies
    this.deleteLocal('cmt', id)
    if (window.navigator.onLine) {
      console.log('Deletando cmt online')
      API.deleteComment(id).then(() => {
        console.log('Deletando hgt online')
        console.log('cmt: ' + id)
        console.log('hgt: ' + hgt)
        this.deleteHighlight(hgt)
      })
    } else {
      this.deleteHighlight(hgt)
    }
  }

  /**
     * Function used save a single object locally.
     * @param {string} type Table in which the object should be saved.
     * @param {object} object Object that should be saved.
     */
  static async saveLocal (type, object) {
    let save = object

    if (type === 'cmt' || type === 'hgt') {
      let current = await this.retrieveLocal(type)
      if (!current) current = []
      object.timestamp = new Date().toJSON()
      console.log(object)
      current.push(object)
      save = current
    }

    if (window.localStorage) {
      window.localStorage.setItem((config.ModuleName.replace(/\s/g, '')) + '_' + type, JSON.stringify(save))
    } else {
      cookies.set((config.ModuleName.replace(/\s/g, '')) + '_' + type, JSON.stringify(save))
    }
  }

  /**
     * Function used save a single object locally.
     * @param {string} type Table in which the object was saved.
     * @return {object} Object returned from table.
     */
  static async retrieveLocal (type) {
    let obj
    if (window.localStorage) {
      obj = window.localStorage.getItem((config.ModuleName.replace(/\s/g, '')) + '_' + type)
    } else {
      obj = cookies.get((config.ModuleName.replace(/\s/g, '')) + '_' + type)
    }
    if (obj) return JSON.parse(obj)
    return false
  }

  /**
     * Function used save the current and maximum page.
     * @param {number} curPage User current page.
     * @param {number} maxPage User maximum page.
     */
  static saveCurMaxPage (curPage, maxPage) {
    const obj = {
      curPage: curPage,
      maxPage: maxPage,
      timestamp: new Date().toJSON()
    }

    this.saveLocal('pageinfo', obj)
    if (window.navigator.onLine) {
      const obj = {
        curPage: curPage,
        maxPage: maxPage
      }
      API.sendPageInfo(obj)
    }
  }

  /**
     * Function used to retrieve pageinformation.
     * @return {object} Object containing the pageinfo.
     */
  static async retrieveCurMaxPage () {
    const local_obj = await this.retrieveLocal('pageinfo')
    let obj = local_obj
    if (window.navigator.onLine) {
      const external_obj = await API.getPageInfo()
      
      if (external_obj) {
        obj = external_obj
        this.saveLocal('pageinfo', obj)
      }
      
    }
    if(obj.maxPage > config.Pages || obj.curPage > config.Pages) {
      obj = {
        maxPage: config.Pages,
        curPage: 1
      }
      this.saveCurMaxPage(1, config.Pages)
    }
    return obj
  }
}
